'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const core = require('@drupal-js-sdk/core');

class DrupalError extends Error {
  constructor(code, message) {
    super(message);
    this.code = code;
    Object.defineProperty(this, "message", {
      enumerable: true,
      value: message
    });
  }
  getErrorCode() {
    return this.code;
  }
  toString() {
    return `DrupalError: ${this.code} ${this.message}`;
  }
}
DrupalError.UNDEFINED = -1;
DrupalError.INTERNAL_SERVER_ERROR = 1;
DrupalError.MISSING_IMPLEMENTATION_ERROR = 42;
DrupalError.CONNECTION_FAILED = 100;
DrupalError.OBJECT_NOT_FOUND = 101;
DrupalError.INVALID_QUERY = 102;
DrupalError.INVALID_CLASS_NAME = 103;
DrupalError.MISSING_OBJECT_ID = 104;
DrupalError.INVALID_KEY_NAME = 105;
DrupalError.INVALID_POINTER = 106;
DrupalError.INVALID_JSON = 107;
DrupalError.COMMAND_UNAVAILABLE = 108;
DrupalError.NOT_INITIALIZED = 109;
DrupalError.INCORRECT_TYPE = 111;
DrupalError.INVALID_CHANNEL_NAME = 112;
DrupalError.PUSH_MISCONFIGURED = 115;
DrupalError.OBJECT_TOO_LARGE = 116;
DrupalError.OPERATION_FORBIDDEN = 119;
DrupalError.CACHE_MISS = 120;
DrupalError.INVALID_NESTED_KEY = 121;
DrupalError.INVALID_FILE_NAME = 122;
DrupalError.INVALID_ACL = 123;
DrupalError.TIMEOUT = 124;
DrupalError.INVALID_EMAIL_ADDRESS = 125;
DrupalError.MISSING_CONTENT_TYPE = 126;
DrupalError.MISSING_CONTENT_LENGTH = 127;
DrupalError.INVALID_CONTENT_LENGTH = 128;
DrupalError.FILE_TOO_LARGE = 129;
DrupalError.FILE_SAVE_ERROR = 130;
DrupalError.DUPLICATE_VALUE = 137;
DrupalError.INVALID_ROLE_NAME = 139;
DrupalError.EXCEEDED_QUOTA = 140;
DrupalError.SCRIPT_FAILED = 141;
DrupalError.VALIDATION_ERROR = 142;
DrupalError.INVALID_IMAGE_DATA = 143;
DrupalError.UNSAVED_FILE_ERROR = 151;
DrupalError.INVALID_PUSH_TIME_ERROR = 152;
DrupalError.FILE_DELETE_ERROR = 153;
DrupalError.FILE_DELETE_UNNAMED_ERROR = 161;
DrupalError.REQUEST_LIMIT_EXCEEDED = 155;
DrupalError.DUPLICATE_REQUEST = 159;
DrupalError.INVALID_EVENT_NAME = 160;
DrupalError.INVALID_VALUE = 162;
DrupalError.USERNAME_MISSING = 200;
DrupalError.PASSWORD_MISSING = 201;
DrupalError.USERNAME_TAKEN = 202;
DrupalError.EMAIL_TAKEN = 203;
DrupalError.EMAIL_MISSING = 204;
DrupalError.EMAIL_NOT_FOUND = 205;
DrupalError.SESSION_MISSING = 206;
DrupalError.MUST_CREATE_USER_THROUGH_SIGNUP = 207;
DrupalError.ACCOUNT_ALREADY_LINKED = 208;
DrupalError.INVALID_SESSION_TOKEN = 209;
DrupalError.MFA_ERROR = 210;
DrupalError.MFA_TOKEN_REQUIRED = 211;
DrupalError.LINKED_ID_MISSING = 250;
DrupalError.INVALID_LINKED_SESSION = 251;
DrupalError.UNSUPPORTED_SERVICE = 252;
DrupalError.INVALID_SCHEMA_OPERATION = 255;
DrupalError.AGGREGATE_ERROR = 600;
DrupalError.FILE_READ_ERROR = 601;
DrupalError.X_DOMAIN_REQUEST = 602;
DrupalError.STORAGE_IN_MEMORY_FAIL = 700;
DrupalError.STORAGE_IN_WEB_FAIL = 701;

class StorageInMemory {
  constructor() {
    this.data = {};
    if (this.isAvailable() === false) {
      throw new DrupalError(DrupalError.STORAGE_IN_MEMORY_FAIL, "Storage is not available.");
    }
  }
  getString(keyName) {
    return this.getItem(keyName);
  }
  setString(keyName, keyValue) {
    this.setItem(keyName, keyValue);
  }
  isAvailable() {
    try {
      const test_string = "__test_data_used_for_checking_storage__";
      this.setString(test_string, test_string);
      const isAvailable = this.getString(test_string) === test_string;
      this.removeItem(test_string);
      return isAvailable;
    } catch (e) {
      return false;
    }
  }
  getItem(keyName) {
    const value = this.data[keyName];
    return value === void 0 ? null : this.data[keyName];
  }
  setItem(keyName, keyValue) {
    this.data[keyName] = keyValue;
  }
  removeItem(keyName) {
    delete this.data[keyName];
  }
  clear() {
    this.set({});
  }
  get() {
    return this.data;
  }
  set(data) {
    this.data = data;
  }
}

class StorageInWeb {
  constructor(getStorage = () => window.localStorage) {
    this.storage = getStorage();
    if (this.isAvailable() === false) {
      throw new DrupalError(DrupalError.STORAGE_IN_WEB_FAIL, "Storage is not available.");
    }
  }
  getString(keyName) {
    return this.storage.getItem(keyName);
  }
  setString(keyName, keyValue) {
    this.storage.setItem(keyName, keyValue);
  }
  isAvailable() {
    try {
      const test_string = "__test_data_used_for_checking_strorage__";
      this.setString(test_string, test_string);
      const isAvailable = this.getString(test_string) === test_string;
      this.removeItem(test_string);
      return isAvailable;
    } catch (e) {
      return false;
    }
  }
  getItem(keyName) {
    const value = this.getString(keyName);
    return value !== null ? JSON.parse(value) : null;
  }
  setItem(keyName, keyValue) {
    const encodedValue = JSON.stringify(keyValue);
    this.setString(keyName, encodedValue);
  }
  removeItem(keyName) {
    return this.storage.removeItem(keyName);
  }
  clear() {
    this.storage.clear();
  }
  get() {
    return Object.keys(this.storage).reduce((obj, k) => {
      return {
        ...obj,
        [k]: this.getItem(k)
      };
    }, {});
  }
  set(data) {
    Object.entries(data).forEach(([key, value]) => this.setItem(key, value));
  }
}

class DrupalMenu {
  constructor(drupal) {
    this.drupal = drupal;
    this.client = drupal.getClientService();
  }
  async getMenu(menuName) {
    return this.getMenuRaw(menuName).then((res) => {
      const data = res.data;
      if (this.checkIfDrupalMenuDataIsValid(data)) {
        const normalizedItems = this.normalizeListItems(data);
        return this.convertFlatListItemsToTree(normalizedItems);
      } else {
        throw new DrupalError(DrupalError.INVALID_JSON, "Menu data is invalid");
      }
    });
  }
  getMenuRaw(menuName) {
    return this.client.call("get", `/system/menu/${menuName}/linkset`);
  }
  normalizeListItems(data) {
    const list = [];
    if (this.checkIfDrupalMenuDataIsValid(data)) {
      const items = data.linkset[0].item;
      items.map((item) => {
        let parentId;
        let level;
        const id = `${item["drupal-menu-machine-name"][0]}${item["drupal-menu-hierarchy"][0]}`;
        const idArray = id.split(".");
        idArray.pop();
        parentId = idArray.join(".");
        level = 0;
        level = idArray.length;
        if (level < 2) {
          parentId = "0";
        }
        const node = {
          id,
          parentId,
          name: item.title,
          href: item.href,
          level,
          items: null
        };
        list.push(node);
      });
    }
    return list;
  }
  checkIfDrupalMenuDataIsValid(data) {
    if (data !== void 0 && data.linkset !== void 0 && data.linkset[0] !== void 0 && data.linkset[0].item !== void 0) {
      return true;
    } else {
      return false;
    }
  }
  convertFlatListItemsToTree(inputList) {
    const roots = [];
    if (inputList.length === 0) {
      return roots;
    }
    const myObjMap = {};
    inputList.map((node, index) => {
      myObjMap[node.id] = index;
      inputList[index].items = [];
    });
    inputList.map((node) => {
      if (node.parentId === "0") {
        roots.push(node);
      } else {
        inputList[myObjMap[node.parentId]].items.push(node);
      }
    });
    return roots;
  }
}

class DrupalAuth {
  constructor(drupal) {
    this.SESSION_KEY = "DRUPAL_AUTH.SESSION";
    this.store = {
      csrfToken: void 0,
      logoutToken: void 0,
      currentUser: {
        uid: "0",
        roles: ["anonymous"],
        name: "Anonymous"
      }
    };
    this.drupal = drupal;
    this.client = drupal.getClientService();
    this.session = this.drupal.getSessionService();
    this.store = this.getDrupalSession();
    this.refreshUserSession();
  }
  getDrupalSession() {
    let sessionData = this.session.getItem(this.SESSION_KEY);
    if (sessionData === null || sessionData === void 0) {
      sessionData = {};
    }
    return sessionData;
  }
  setDrupalSession() {
    this.session.setItem(this.SESSION_KEY, this.store);
  }
  refreshUserSession() {
    return this.getSessionToken();
  }
  getSessionToken() {
    const config = {
      headers: {
        "Content-Type": "application/json"
      },
      withCredentials: true
    };
    return this.client.call("get", "/session/token", config).then((response) => {
      const data = response.data;
      this.store.csrfToken = data;
      this.client.addDefaultHeaders({ "X-CSRF-Token": data });
      return response;
    });
  }
  login(name, pass) {
    const config = {
      headers: {
        "Content-Type": "application/json"
      },
      withCredentials: true,
      params: {
        _format: "json"
      },
      data: {
        name,
        pass
      }
    };
    return this.client.call("post", "/user/login", config).then((response) => {
      const data = response.data;
      this.store = data;
      this.setDrupalSession();
      this.client.addDefaultHeaders({ "X-CSRF-Token": data.csrfToken });
      return response;
    });
  }
  loginStatus() {
    const config = {
      headers: {
        "Content-Type": "application/json"
      },
      withCredentials: true,
      params: {
        _format: "json"
      }
    };
    return this.client.call("get", "/user/login_status", config).then((response) => {
      return response.data !== 0;
    });
  }
  forcedLogout() {
    if (this.store.logoutToken) {
      return this.logout().then(() => true);
    }
    const config = {
      withCredentials: true
    };
    return this.client.call("get", "/user/logout", config).then(() => true);
  }
  logout() {
    const config = {
      headers: {
        "Content-Type": "application/json"
      },
      withCredentials: true,
      params: {
        _format: "json",
        token: this.store.logoutToken
      }
    };
    return this.client.call("post", "/user/logout", config).then((response) => {
      return response;
    });
  }
  passwordResetByUserName(name) {
    const config = {
      headers: {
        "Content-Type": "application/json"
      },
      withCredentials: true,
      params: {
        _format: "json"
      },
      data: {
        name
      }
    };
    return this.client.call("post", "/user/password", config).then((response) => {
      return response;
    });
  }
  passwordResetByMail(mail) {
    const config = {
      headers: {
        "Content-Type": "application/json"
      },
      withCredentials: true,
      params: {
        _format: "json"
      },
      data: {
        mail
      }
    };
    return this.client.call("post", "/user/password", config).then((response) => {
      return response;
    });
  }
  register(name, mail) {
    const config = {
      headers: {
        "Content-Type": "application/json"
      },
      withCredentials: true,
      params: {
        _format: "json"
      },
      data: {
        name: {
          value: name
        },
        mail: {
          value: mail
        }
      }
    };
    return this.client.call("post", "/user/register", config).then((response) => {
      return response;
    });
  }
}

exports.DrupalAuth = DrupalAuth;
exports.DrupalError = DrupalError;
exports.DrupalMenu = DrupalMenu;
exports.StorageInMemory = StorageInMemory;
exports.StorageInWeb = StorageInWeb;
for (const k in core) {
  if (k !== 'default' && !exports.hasOwnProperty(k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return core[k]; }
  });
}
